<?php

/*
 * Copyright (C) 2011 Alejandro Albarca Martínez <albarcam [arroba] gmail.com> and NETFLIE. (http://www.netflie.es)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program, in the file called "COPYING".  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The class is responsible of getting, setting and filtering the possible values of the used measurement units used by the station sensors.
 * @name       Unit
 * @package    PHPWeatherStation
 * @copyright  Copyright (c) 2011 NETFLIE. (http://meteolive.netflie.es)
 * @license    http://www.gnu.org/licenses/     GNU GPLv3
 * @version    $Id: Units.php 2011-10-17 17:00
 * @author     Alejandro Albarca Martinez
 */
class Unit {

  /**
   * Date unit
   * Allowed values: 'local' or 'utc'
   * @var array
   */
  private $_date;
  
  /**
   * Humidity unit
   * Allowed values: 'abs' or 'rel'
   * @var array
   */
  private $_hum;

  /**
   * Pressure unit
   * Allowed values: 'hpa, 'inhg', 'mmhg' or 'psi'
   * @var array
   */
  private $_pres;

  /**
   * Temp unit
   * Allowed values: 'c' or 'f'
   * @var array
   */
  private $_temp;

  /**
   * Rain unit
   * Allowed values: 'in' or 'mm'
   * @var array
   */
  private $_rain;

  /**
   * Wind unit
   * Allowed values: 'ms', 'kmh', 'mph' or 'kn'
   * @var array
   */
  private $_wind;

  /**
   * Create a new Unit object. 
   * @example examples/newUnitObject.example Creating a new Unit object
   * 
   * @param array $units Array of units.
   */
  public function __construct($units) {
    try {
      if ($units) {
        if (isset($units[0]) && isset($units[1]) && isset($units[2]) && isset($units[3]) && isset($units[4]) && isset($units[5])
                || isset($units['U_DATE']) && isset($units['U_HUM']) && isset($units['U_PRES']) && isset($units['U_TEMP']) && isset($units['U_RAIN']) && isset($units['U_WIND'])) {

          $this->__set('date', isset($units[0]) ? $units[0] : $units['U_DATE']);
          $this->__set('hum' , isset($units[1]) ? $units[1] : $units['U_HUM'] );
          $this->__set('pres', isset($units[2]) ? $units[2] : $units['U_PRES']);
          $this->__set('temp', isset($units[3]) ? $units[3] : $units['U_TEMP']);
          $this->__set('rain', isset($units[4]) ? $units[4] : $units['U_RAIN']);
          $this->__set('wind', isset($units[5]) ? $units[5] : $units['U_WIND']);
        } else {
          throw new Exception('Unit constructor is not valid. Review the documentation.');
        }
      }
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Set a value for a unit.
   * 
   * @param array $name Unit name
   * @param arry $value Unit value
   */
  public function __set($name, $value) {
    $name = strtolower($name);
    $value = strtolower($value);

    $nameUnit = '_' . $name; // Private name var
    try {
      // If the unit exist
      if (property_exists($this, $nameUnit)) {
        // If the unit value is valid
        if ($this->_isValid($name, $value)) {
          $this->$nameUnit = $value;
        } else {
          throw new Exception('The value: "' . $value . '" for "' . $name . '" unit, is not valid.');
        }
      } else {
        throw new Exception('The unit "' . $name . '" does not exist.');
      }
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Get the value of a unit.
   * 
   * @param array $name Unit name
   * @return array Unit value
   */
  public function __get($name) {
    $name = strtolower($name);
    
    $pattern = '/(date|dew|heatindex|hum|humidex|pres|sealevel|temp|rain|wind)/';
    preg_match($pattern, $name, $matches);

    if ($matches && $matches[1] == 'date')
      return $this->_date;
    else if ($matches &&  $matches[1] == 'hum')
      return $this->_hum;
    else if ($matches && ($matches[1] == 'pres' || $matches[1] == 'sealevel'))
      return $this->_pres;
    else if ($matches && ($matches[1] == 'temp' || $matches[1] == 'dew' || $matches[1] == 'heatindex' || $matches[1] == 'humidex'))
      return $this->_temp;
    else if ($matches && ($matches[1] == 'rain'))
      return $this->_rain;
    else if ($matches && ($matches[1] == 'wind'))
      return $this->_wind;
    try {
      throw new Exception('The unit "' . $name . '" does not exist.');
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Determine if the value for a unit is valid or not.
   * @param array $name Unit name
   * @param array $value Unit value
   * @return bool Return true if the value for the unit is valid.
   */
  private function _isValid($name, $value) {
    switch ($name) {
      case 'date':
        $pattern = '/^(local|utc)$/';
        break;
      case 'hum':
        $pattern = '/^(abs|rel)$/';
        break;
      case 'pres':
        $pattern = '/^(hpa|inhg|mmhg|psi)$/';
        break;
      case 'temp':
        $pattern = '/^(c|f)$/';
        break;
      case 'rain':
        $pattern = '/^(in|mm)$/';
        break;
      case 'wind':
        $pattern = '/^(ms|kmh|mph|kn)$/';
        break;
    }

    if (preg_match($pattern, $value))
      return true;

    return false;
  }

}